(* ---------------------------------------------------------------- 
   Eine Resource-Datei als als Object 
   ---------------------------------------------------------------- *)

Unit MDialog;

Interface

Uses MRSRC, M_DIALS, M_TYPES;

Type	PDialog  = ^TDialog;
		TDialog	 = Object
			Rsc			: PtrRsrc;
			Tree		: Pointer;
			Mode		: DialMode;
			Window		: Integer;
			IsOpen		: Boolean;
			Handler		: DialHandler;
			DialDone	: Boolean;
			InitOk		: Boolean;
			
			Constructor	Init( Rsrc		: PtrRsrc; 
							  Index 	: Integer;
							  MyHandler	: DialHandler );
			
			Procedure	Info( Var Md : DialMode; Var Win : Integer;
							  Var Open, OwnApplication	 : Boolean );

			Procedure	OpenEdit( Md			: DialMode;
								  Flags			: LongBitSet;
								  Edit, EditPos : Integer;
								  Ok, Cancel	: Integer;
								  Xpos, YPos	: Integer;
								  Titel			: String );

			Procedure	OpenNoEdit( Md			: DialMode;
								  Flags			: LongBitSet;
								  Ok, Cancel	: Integer;
								  Xpos, YPos	: Integer;
								  Titel			: String );

			Procedure	NormHandler( P : PtrDialPara );	Virtual;
			Procedure	ResetButton( P : PtrDialPara ); Virtual;

			Procedure	Run;		Virtual;
			Procedure	Fin( P : PtrDialPara );	Virtual;
			Procedure	ClrBoxen( Boxen : Array of Integer );	Virtual;
			Procedure	ClrEdits( Edits : Array of Integer );	Virtual;
			Procedure	RedrawList( P : PtrDialPara; Elements : Array of Integer ); 	Virtual;

			Procedure	PutStr( Index : Integer; s : String );	Virtual;
			Procedure	GetStr( Index : Integer; Var S : String ); Virtual;

			Procedure	PutInt( Index, Value : Integer ); Virtual;
			Procedure	GetInt( Index : Integer; Var Value : Integer ); Virtual;

			Procedure	PutReal( Index : Integer; Value : Real ); Virtual;
			Procedure	GetReal( Index : Integer; Var Value : Real ); Virtual;

			Function	GetString( Index : Integer ) : String;	Virtual;
			Function	GetInteger( Index : Integer ) : Integer; Virtual;
			Function	GetFloat( Index : Integer ) : Real; Virtual;

			
			
		End;

Implementation

Uses MyString, M_Messages, m_alerts, m_windows, m_event, m_convert;

Const AlertStr = '[1|Information][Object-Dialogs unter|%FMAGIC 400][[Ok]';

Procedure TDialog.Info;
var Top	 : Integer;
	My	 : Boolean;
Begin
	DialInfo( Tree, Mode, Window, IsOpen );
	GemTop( Top, My );
	Win 	:= Window;
	Open	:= IsOpen;
	OwnApplication	:= IsOpen and My And ( Top = Window );
End;

Procedure   TDialog.Fin;
Begin
	DialDone := True;
	p^.Action := [aClose];
End;

Procedure	TDialog.NormHandler;
Begin
	p^.Action := [];
	if Mode = dNonModal Then
	Begin
		if p^.Message = WinClosed Then Fin( p );
	End Else
	Begin
		If p^.Message =OutsideClick Then Alert( 0, 0, 1, AlertStr );
	End;
End;

Constructor	TDialog.Init;
Begin
	Rsc := Rsrc;
	Tree:= Rsc^.GetTree( Index );
	Handler := MyHandler;
	InitOk  := NewDial( Tree, Handler );
End;

Procedure	TDialog.OpenEdit;
var d1 : DialMode;
	w1 : Integer;
	b1, b2 : Boolean;
Begin
	If InitOk Then
	Begin
		OpenDial( 	Tree, Md, Flags, 
					Edit, EditPos, Ok, Cancel, XPos, YPos,
				  	Titel );
		DialDone := False;
		Info( D1, W1, B1, B2 );
	End;
End;

Procedure	TDialog.OpenNoEdit;
Begin
	OpenEdit( Md, Flags, -1, -1, Ok, Cancel, XPos, YPos, Titel );
End;

Procedure TDialog.ResetButton;
Begin
	P^.Action := [aDeselect,aDraw];
End;

Procedure	TDialog.Run;
Begin
	Repeat
		CentralDispatcher;
	Until DialDone or Not IsOpen;
End;

Procedure	TDialog.ClrBoxen;
var i : Integer;
Begin
	i := 0;
	While Boxen[i] > -1 Do
	Begin
		FreeListbox( Tree, Boxen[i] );
		inc( i );
	End;
End;

Procedure	TDialog.ClrEdits;
var i : Integer;
Begin
	i := 0;
	While Edits[i] > -1 Do
	Begin
		PutStr( Edits[i], '' );
		inc( i );
	End;
End;

Procedure	TDialog.PutStr;
Begin
	mystring.putstr( Tree, Index, s );
End;

Procedure	TDialog.GetStr;
Begin
	mystring.getstr( Tree, Index, s );
End;

Function	TDialog.GetString;
var s : String;
Begin
	GetStr( Index, s );
	GetString := s;
End;

Procedure Tdialog.PutInt;
Begin
	mystring.putint( Tree, Index, Value );
End;

Procedure Tdialog.GetInt;
Begin
	mystring.Getint( Tree, Index, Value );
End;

Function TDialog.GetInteger;
var i : Integer;
Begin
	GetInt( Index, i );
	GetInteger := i;
End;

Procedure	TDialog.PutReal;
var s : Str40;
Begin
	Str( Value, s );
	PutStr( Index, s );
End;

Procedure	TDialog.GetReal;
var s : Str40;
Begin
	GetStr( Index, s );
	Value := StrToReal( s );
End;

Function	Tdialog.GetFloat;
var value : Real;
Begin
	GetReal( Index, Value );
	GetFloat := Value;
End;

Procedure TDialog.RedrawList;
var i, j : Integer;
Begin
	i := 0;
	j := 0;
	While Elements[i] < -1 Do
	Begin
		p^.List^[j] := Elements[i];
		inc( j );
		p^.List^[j] := 1;
		Inc( j );
		inc( i );
	End;
	p^.List^[j] := -1;
	p^.Action := P^.Action + [aDrawList];
End;

End.
