(* ---------------------------------------------------------------- 
   Eine Resource-Datei als als Object 
   ---------------------------------------------------------------- *)

Unit MRSRC;

Interface

Uses Dos, M_Rsc;

Type 
	TRscType = ( _RscEmpty, _RscLoaded, _RscRelocated );

	
	PtrRsrc  = ^TRsrc;
	TRsrc	  = Object
		Rsc			: Pointer;		{ Zeiger auf die Resourceparameter }
		RscType		: TRscType;		{ Resourcen geleaden oder Speicherresident }
		RscInfo		: ^PathStr;		{ Adresse bzw. Filename der Resourcen }
		Relocated	: Boolean;		{ TRUE wenn erfolgreich geladen und/oder reloziert } 

		{ Constructoren und Destructor }
		Constructor	Init( PData : Pointer );
		Constructor	Load( FName : PathStr );
		Destructor	Done;

		{ Abfrage- und Manipulationsmethoden }
		Function	RscName : PathStr;
		Function	GetTree( Index : Integer ) : Pointer;
		Function	GetAddr( RType, Index : Integer ) : Pointer;
		Procedure	SetAddr( RType, Index : Integer; Tree : Pointer );
		Procedure	FixObject( Tree : Pointer; Index : Integer );

	Private
		Procedure	Clear;
	End;
		
Implementation

Uses M_AES;

{ Einen Speicherresidenten Resource relozieren }
Constructor	TRsrc.Init;
Begin
	Clear;
	RscInfo		:= PData;
	Relocated	:= RelocRsc( Pointer( RscInfo ), Rsc );
	if Relocated Then RscType := _RscRelocated;
End;

{ Einen Externen Resource Laden }
Constructor TRsrc.Load;
var S : PathStr;
Begin
	Clear;
	S			:= Fexpand( FName );
	GetMem( RscInfo, Length(s) + 2 );
	RscInfo^	:= s;
	Relocated	:= LoadRsc( s, Rsc );
	if Relocated Then RscType := _RscLoaded;
End;

{ Resouce freigeben }
Destructor	TRsrc.Done;
Begin
	If RscType = _RscLoaded Then
	Begin
		FreeRsc( Rsc );
		FreeMem( RscInfo, Length( RscInfo^ ) + 2 );
	End;
	Clear;
End;

{ Interne Lschroutiene }
Procedure	TRsrc.Clear;
Begin
	Fillchar( Self, Sizeof( TRsrc ), 0 );
End;

{ Mamen der Compilierten Resourcedatei Freigeben }
Function	TRsrc.RscName;
Begin
	case RscType of
		_RscEmpty:		RscName := '*EMPTY*';
		_RscRelocated:	RscName := '*INTERN*';
		_RscLoaded:		RscName := RscInfo^;
	End;
End;

{ Adresse eines Objectbaums Ermittlen }
Function TRsrc.GetTree;
Begin
	GetTree := GAddrRsc( Rsc, RTREE, Index );
End;

{ Einzenes Object innerhalb eines Trees ermitteln }
Function TRsrc.GetAddr;
Begin
	GetAddr := GAddrRsc( Rsc, RType, Index );
End;

Procedure TRsrc.SetAddr;
Begin
	SaddrRsc( Rsc, RType, Index, Tree );
End;

{ Object vom Resource-Format ins AES-Format umrechnen }
Procedure TRsrc.FixObject;
Begin
	ObFixRsc( Rsc, Tree, Index );	
End;

End.
