(*----------------------------------------------------------------------*
 *                                                                      *
 *  MAGIC        Modular  Applications  GEM  Interface  Cadre           *
 *                                                                 *
 *----------------------------------------------------------------------*
 * Version 4.00  01.05.1994   (C)1990..1994 by Peter Hellinger Software *
 *----------------------------------------------------------------------*
 *            Dieses Modul ist urheberrechtlich geschtzt.              *
 *                                                                      *
 * Die Verffentlichung des Quelltextes oder Teilen daraus, sowie die   *
 * Verbreitung des bersetzten, nicht gelinkten Codes in schriftlicher, *
 * oder maschinenlesbarer Form, insbesondere in Zeitschriften, Mail-    *
 * boxen oder anderen Medien bedarf der ausdrcklichen schriftlichen    *
 * Einverstndnisserklrung des Autors.                                 *
 *                                                                      *
 * Die Verbreitung des Moduls als Teil eines gelinkten Programms ist    *
 * fr Lizenznehmer ausdrcklich erlaubt!  Der Autor behlt sich das    *
 * Recht vor, diese Erlaubnis jederzeit und ohne Angaben von Grnden    *
 * zu widerrufen.                                                       *
 *----------------------------------------------------------------------*)
 
UNIT M_Appl;

INTERFACE

USES M_Types;

VAR     ApplIdent:      INTEGER;
        AESVersion:     INTEGER;
        AESFontid:      INTEGER;
        AESFontsize:    INTEGER;
        CharWidth:      INTEGER;
        CharHeight:     INTEGER;
        BoxWidth:       INTEGER;
        BoxHeight:      INTEGER;
        DeskX:          INTEGER;
        DeskY:          INTEGER;
        ScreenRect:     tRect;
        Win0Rect:       tRect;
        Multitask:      BOOLEAN;
        KaosTos:        BOOLEAN;
        MiNTInstalled:  BOOLEAN;
        MiNTVersion:    LONGINT;
        SpeedoGDOS:     BOOLEAN;
        SpeedoVersion:  INTEGER;
        DefaultWS:      WORKSTATION;

PROCEDURE ApplInit;

TYPE TermProc = FUNCTION: INTEGER;

PROCEDURE InstallTermproc (proc: TermProc);
PROCEDURE ApplTerm (return: INTEGER);
PROCEDURE ApplPath (VAR path: STRING);
PROCEDURE ApplName (VAR name: STRING);

TYPE Device = (Screen,    (* Device-Nummer  1..10 *)
               Plotter,   (*               11..20 *)
               Printer,   (*               21..30 *)
               Metafile,  (*               31..40 *)
               Camera,    (*               41..50 *)
               Tablett,   (*               51..60 *)
               Memory,    (*               61..70 *)
               dev71_80,  (*               61..70 *)
               dev81_90,  (*               71..80 *)
               dev91_99); (*               81..90 *)

PROCEDURE OpenWorkstation (device: Device; devnum, w, h: INTEGER; rc: BOOLEAN; VAR wsinfo: WORKSTATION);
PROCEDURE CloseWorkstation (VAR wsinfo: WORKSTATION);
FUNCTION  AvailableDrivers (dev: Device): INTEGER;
PROCEDURE DriverInfo (dev: Device; num: INTEGER; VAR wsinfo: WORKSTATION);
FUNCTION VqGdos: LONGINT;
PROCEDURE MouseOn;
PROCEDURE MouseArrow;
PROCEDURE MouseCursor;
PROCEDURE MouseBusy;
PROCEDURE MouseFinger;
PROCEDURE MouseHand;
PROCEDURE MouseThincross;
PROCEDURE MouseThickcross;
PROCEDURE MouseOutline;
PROCEDURE MouseUser;
PROCEDURE MouseRotor;
PROCEDURE MouseOff;
PROCEDURE MouseOffRect (r: tRect);
PROCEDURE UserMouse (VAR form: MFORM);
PROCEDURE MouseState (VAR form: INTEGER; VAR sichtbar: BOOLEAN);
PROCEDURE StoreMouse;
PROCEDURE RestoreMouse;
PROCEDURE SetMouse (x, y: INTEGER);
PROCEDURE LockScreen;
PROCEDURE UnlockScreen;
PROCEDURE FreezeScreen;
PROCEDURE UnfreezeScreen;
FUNCTION  ScreenLocked: BOOLEAN;
FUNCTION  GetHelp (CONST txt: ARRAY OF CHAR): INTEGER;

IMPLEMENTATION

{$I COMPOPT.INC}

USES M_Sys, M_Cookie, M_Strings, M_AES, M_VDI;

CONST   ABCGEM =        $0210;

CONST   cNDC =           0;
        cRC =            2;

TYPE    TermPtr =       ^TermInfo;
        TermInfo =      RECORD
                         proc:  TermProc;
                         next:  TermPtr;
                        END;


VAR     vapplpath:      ARRAY [0..255] OF CHAR;
        vapplname:      ARRAY [0..255] OF CHAR;

VAR     ProgReturn:     INTEGER;
        init:           INTEGER;
        gdos:           LONGINT;


PROCEDURE ApplInit;
VAR b: BOOLEAN;
    l: LONGINT;
    name: STRING;
    attr: tTextAttrArray;
    a: Anytype;
BEGIN
 IF init <> 30961 THEN BEGIN
  
  ApplIdent:= appl_init;
  AESVersion:= App^.AESPB^.cbPGlobal^.apVersion;
  
  CallMagic (1);    (* Magic initialisieren *)
  CallMagic (609);  KaosTos:= BOOLEAN (App^.MagicPB^.IntOut[0]);
  CallMagic (608);  Multitask:= BOOLEAN (App^.MagicPB^.IntOut[0]);

  MiNTInstalled:= FindCookie (MiNT, a);
  MiNTVersion:= a.long;
 
  CallMagic (604);
  AESFontid:=     App^.MagicPB^.IntOut[0];
  AESFontsize:=   App^.MagicPB^.IntOut[1];
  Charwidth:=     App^.MagicPB^.IntOut[2];
  Charheight:=    App^.MagicPB^.IntOut[3];
  Boxwidth:=      App^.MagicPB^.IntOut[4];
  Boxheight:=     App^.MagicPB^.IntOut[5];
  
  CallMagic (644);
  SpeedoGDOS:=    BOOLEAN (App^.MagicPB^.IntOut[0]);
  SpeedoVersion:= App^.MagicPB^.IntOut[1];

  App^.MagicPB^.AddrIn[0]:= ADDR (ScreenRect);
  CallMagic (605);

  App^.MagicPB^.AddrIn[0]:= ADDR (Win0Rect);
  CallMagic (606);

  OpenWorkstation (Screen, -1, 0, 0, TRUE, DefaultWS);
 
  CallMagic (607);
  DeskX:=   App^.MagicPB^.IntOut[0];
  DeskY:=   App^.MagicPB^.IntOut[1];

  IF AESVersion <> ABCGEM THEN  gdos:= M_Sys.VqGdos()
                          ELSE  gdos:= ABCGEM;

  Exitproc:= ADDR (ApplTerm);

  CallMagic (621);
  
 END;
END (*ApplInit*);

PROCEDURE InstallTermproc (proc: TermProc);
VAR p: TermPtr;
BEGIN
 GetMem (p, SIZEOF (TermPtr));  IF p = NIL THEN  HALT;
 p^.proc:= proc; 
 p^.next:= App^.TermProc;
 App^.TermProc:= p;
END (*InstallTermproc*);

PROCEDURE ApplTerm (return: INTEGER);
VAR  p: TermPtr;
     i: INTEGER;
BEGIN
 (* Installierte TermProcedures abarbeiten *)
 p:= App^.TermProc;
 WHILE (p <> NIL) DO BEGIN  (*i:=*) p^.proc;  p:= p^.next;  END;
 App^.MagicPB^.IntIn[0]:= return;
 CallMagic (4);   (* Alles deinstallieren *)
 CallMagic (2);   (* ExitMagic *)
 Terminate (return);
END (*ApplTerm*);

PROCEDURE ApplPath (VAR path: STRING);
BEGIN
 path:= StrPas (vapplpath);
END (*ApplPath*);

PROCEDURE ApplName (VAR name: STRING);
BEGIN
 name:= StrPas (vapplname);
END (*ApplName*);

PROCEDURE OpenWorkstation (device: Device; devnum, w, h: INTEGER; rc: BOOLEAN;
                           VAR wsinfo: WORKSTATION);
BEGIN
 App^.MagicPB^.AddrIn[0]:= ADDR (wsinfo);
 App^.MagicPB^.IntIn[0]:=  ORD (device);
 App^.MagicPB^.IntIn[1]:=  devnum;
 App^.MagicPB^.IntIn[2]:=  w;
 App^.MagicPB^.IntIn[3]:=  h;
 App^.MagicPB^.IntIn[4]:=  ORD (rc);
 CallMagic (602);
END (*OpenWorkstation*);

PROCEDURE CloseWorkstation (VAR wsinfo: WORKSTATION);
BEGIN
 App^.MagicPB^.AddrIn[0]:= ADDR (wsinfo);
 CallMagic (603);
END (*CloseWorkstation*);

FUNCTION  AvailableDrivers (dev: Device): INTEGER;
BEGIN
 App^.MagicPB^.IntIn[0]:= ORD (dev);
 CallMagic (642);
 AvailableDrivers:= App^.MagicPB^.IntOut[0];
END (*AvailableDrivers*);

PROCEDURE DriverInfo (dev: Device; num: INTEGER; VAR wsinfo: WORKSTATION);
BEGIN
 App^.MagicPB^.AddrIn[0]:= ADDR (wsinfo);
 App^.MagicPB^.IntIn[0]:= ORD (dev);
 App^.MagicPB^.IntIn[1]:= num;
 CallMagic (643);
END (*DriverInfo*);

PROCEDURE MouseOn;
BEGIN
 CallMagic (620);
END (*MouseOn*);

PROCEDURE MouseArrow;
BEGIN
 CallMagic (621);
END (*MouseArrow*);

PROCEDURE MouseCursor;
BEGIN
 CallMagic (622);
END (*MouseCursor*);

PROCEDURE MouseBusy;
BEGIN
 CallMagic (623);
END (*MouseBusy*);

PROCEDURE MouseFinger;
BEGIN
 CallMagic (624);
END (*MouseFinger*);

PROCEDURE MouseHand;
BEGIN
 CallMagic (625);
END (*MouseHand*);

PROCEDURE MouseThincross;
BEGIN
 CallMagic (626);
END (*MouseThincross*);

PROCEDURE MouseThickcross;
BEGIN
 CallMagic (627);
END (*MouseThickcross*);

PROCEDURE MouseOutline;
BEGIN
 CallMagic (628);
END (*MouseOutline*);

PROCEDURE MouseRotor;
BEGIN
 CallMagic (640);
END (*MouseRotor*);

PROCEDURE MouseUser;
BEGIN
 CallMagic (629);
END (*MouseUser*);

PROCEDURE MouseOff;
BEGIN
 CallMagic (630);
END (*MouseOff*);

PROCEDURE UserMouse (VAR form: MFORM);
BEGIN
 App^.MagicPB^.AddrIn[0]:= ADDR (form);
 CallMagic (631);
END (*UserMouse*);

PROCEDURE MouseState (VAR form: INTEGER; VAR sichtbar: BOOLEAN);
BEGIN
 CallMagic (632);
 form:= App^.MagicPB^.IntOut[0];
 sichtbar:= BOOLEAN (App^.MagicPB^.IntOut[0]);
END (*MouseState*);

PROCEDURE StoreMouse;
BEGIN
 CallMagic (633);
END (*StoreMouse*);

PROCEDURE RestoreMouse;
VAR view: BOOLEAN;
BEGIN
 CallMagic (634);
END (*RestoreMouse*);

PROCEDURE SetMouse (x, y: INTEGER);
VAR xx, yy: INTEGER;
    ch:     CHAR;
    s:      BITSET;
BEGIN
 App^.MagicPB^.IntIn[0]:= x;
 App^.MagicPB^.IntIn[1]:= y;
 CallMagic (635);
END (*SetMouse*);

PROCEDURE MouseOffRect (r: tRect);
BEGIN
 App^.MagicPB^.AddrIn[0]:= ADDR (r);
 CallMagic (641);
END (*MouseOffRect*);

FUNCTION VqGdos (): LONGINT;
BEGIN 
 VqGdos:= gdos;
END (*VqGdos*);

FUNCTION NoMemory (size: LONGINT): INTEGER;
BEGIN
 NoMemory:= 1;  (* NIL bei Speichermangel liefern *)
END (*NoMemory*);

PROCEDURE LockScreen;
BEGIN
 CallMagic (636);
END (*LockScreen*);

PROCEDURE UnlockScreen;
BEGIN
 CallMagic (637);
END (*UnlockScreen*);

PROCEDURE FreezeScreen;
BEGIN
 CallMagic (638);
END (*FreezeScreen*);

PROCEDURE UnfreezeScreen;
BEGIN
 CallMagic (639);
END (*UnfreezeScreen*);

FUNCTION ScreenLocked: BOOLEAN;
BEGIN
 ScreenLocked:= M_Types.ScreenLocked IN App^.flags;
END (*ScreenLocked*);

PROCEDURE InitMtAppl;
VAR s: ARRAY [0..255] OF CHAR;
    c, d, l, i: INTEGER;
    p1: PCHAR;
BEGIN
 IF init <> 30961 THEN BEGIN
  ApplInit;  (* Applikation und Workstations initialisieren *)
  shel_read (vapplpath, s);
  l:= StrLen (vapplpath);  c:= l;  d:= 1;
  WHILE ((c > 1) AND (vapplpath[c-1] <> '\')) DO BEGIN
   DEC (c);  INC (d);
  END; (* WHILE *)
  p1:= ADDR(vapplpath[c]);
  StrLCopy (vapplname, p1^, d);
  SYSTEM.TermProc:= ADDR (ApplTerm);
  SYSTEM.HeapError:= ADDR (NoMemory);
  init:= 30961;
 END;
END (*InitMtAppl*);

FUNCTION  GetHelp (CONST txt: ARRAY OF CHAR): INTEGER;
BEGIN
 App^.MagicPB^.AddrIn[0]:= ADDR (txt);
 CallMagic (645);
 GetHelp:= App^.MagicPB^.IntOut[0];
END (*GetHelp*);

BEGIN
 init:= 0;
 InitMtAppl;
END (*M_Appl*).
