(*----------------------------------------------------------------------*
 * MagicCookie  Zugriff auf den Cookiejar                               *
 *                                                                      *
 * Cookies sind, hnlich XBRA im System global zugngliche Variablen.   *
 * Sie bestehen jeweils aus einem Namen (4 Byte, mglichst ein lesbarer *
 * Text) und einem 32Bit-Wert. Sie werden vom Betriebssystem (ab TOS-   *
 * Version 1.06 oder von residenten Programmen eingetragen.             *
 *                                                                      *
 * Sie dienen dazu, eine statische Schnittstelle zu anderen Programmen  *
 * herzustellen. Wenn beispielsweise ein residentes Programm (im AUTO-  *
 * Ordner) einen Cookie anlegt, dessen Wert ein Zeiger auf einen eige-  *
 * nen Datenbereich ist, knnen andere Programme spter diesen Cookie   *
 * finden und auf die Daten des residenten Programms zugreifen. Atari   *
 * hat ausserdem einige Cookies definiert, die den Rechner und die      *
 * Hardware beschreiben.                                                *
 *                                                                      *
 * VORSICHT: Wenn Sie selbst eigene Cookies eintragen wollen, mssen    *
 * Sie einiges mehr an Wissen darber haben (z.B. Atari STE TOS Release *
 * Notes vom 12.1.90). Vor allem sollten Sie Cookies nur in residenten  *
 * Programmen erzeugen, jedoch NIE in Accessories oder Applikationen,   *
 * ohne sich ber die Problematik klar zu sein.                         *
 *----------------------------------------------------------------------*
 *                                                                      *
 * Hinweis fr die Pascal-Implementation:                               *
 * Die Funktionen erwarten als Cookie-ID einen PASCAL-String, also mit  *
 * fhrendem Lngenbyte! Intern wird auf einen C-String gewandelt.      *
 *----------------------------------------------------------------------*)

UNIT M_Cookie;

INTERFACE

(* Von Atari definierte Cookies, die den Rechner beschreiben. *)

CONST   CPU =           '_CPU'; (* Hauptprozessor: 0, 10, 20, 30 usw. *)
        FPU =           '_FPU'; (* Mathe-Koprozessor: 0, 1, 2 im HighWord *)
        Machine =       '_MCH'; (* Rechner; HighWord: 0=ST, 1=STE, 2=TT *)
        Video =         '_VDO'; (* Video-Hardware; HighWord: 0=ST, 1=STE, 2=TT *)
        Sound =         '_SND'; (* Bits im Longword: 0: GI/Yamaha(ST), 1: DMA(TT) *)
        Switches =      '_SWI'; (* BIOS-interne Konfigurationswerte *)
        FastReadBuff =  '_FRB'; (* Adresse des 64KB-Puffers f. Disk-Treiber im TT *)
        LaserPrinter =  '_SLM'; (* Wird vom Diablo-Treiber (ab Version 1.4) gesetzt. *)
        STeFix =        '_INF'; (* Fix fr den Desktop.Inf-Bug im Desktop des STE. *)
        DosFlock =      '_FLK'; (* GEMDOS verfgt ber Record-Locking *)
        Network =       '_NET'; (* Typ des verwendeten Netzwerks *)
        Poolfix =       '_OOL'; (* Poolfix 3 von Atari Corp *)
        DayTime =       '_IDT'; (* Datums und Zeitformat *)
        HdDisk =        '_FDC'; (* Laufwerk-Konfiguration ab TOS 2.06
                                 * im 1. Word  0 =  DD-Laufwerke (720/360kB)
                                 *             1 =  HD-Laufwerke (1.44MB)
                                 *             2 =  ED-Laufwerke (2.88MB)
                                 *)

(* Andere bekannte Cookies *)

CONST   Overscan =      'OVER'; (* Autoswitch-Overscan von K.Isakovic *)
        TOS14Fix =      'TFIX'; (* Tos14Fix *)
        VDIFix =        'VFIX'; (* VDIFix-Progamm von K.Isakovic *)
        ReflexCard =    'RFLX'; (* Treiber der Reflex-Karte von OverScan GbR *)
        NVDI =          'NVDI'; (* NVDI von Bela *)
        BigScreen2 =    'BIGS'; (* Grobildschirmsimulator von Julian Reschke *)
        VirtualScreen = 'VSCR'; (* Zeiger auf Infostruktur von Bigscreen2 *)
        Gemini =        'GMNI'; (* GEMINI, der Shareware-Desktop *)
        Mupfel =        'MUPF'; (* MUPFEL von Gereon Steffens *)
        MakeJar =       'MkJr'; (* MakeCookiejar von Thomas Schallar *)
        TTTF =          'TTTF'; (* ThirtyTwoToTwentyFour von Marcel Waldvogel *)
        FileSelector =  'FSEL'; (* Extended Fileselector z.B. Patzelbox *)
        LetemFly =      'LTMF'; (* Let'em Fly von Oliver Scheel *)
        MiNT =          'MiNT'; (* MiNT *)
        MagX =          'MagX'; (* MagX von Behne Bros. *)
        Magic =         'MPRO'; (* Magic 4.00 *)

(* Zum Zerlegen des Cookiewertes ohne groe Umstnde *)
TYPE    Anytype =  RECORD
                    CASE INTEGER OF
                     0: (ptr: POINTER);
                     1: (long: LONGINT);
                     2: (hi, lo: INTEGER);
                     3: (b4, b3, b2, b1: BYTE);
                   END;

FUNCTION FindCookie (id: STRING; VAR value: Anytype): BOOLEAN;
(* Liefert TRUE, wenn der Cookie 'id' gefunden wird. In 'value' 
 * steht der dazugehrige Longword-Wert, bei FALSE ist 'value' 
 * immer Null.
 *)

FUNCTION ModifyCookie (id: STRING; newvalue: Anytype): BOOLEAN;
(* Sucht den Cookie 'id' und liefert FALSE, wenn er nicht vorhanden 
 * ist. Wird er gefunden, wird 'value' als neuer Cookie-Wert einge-
 * setzt und die Funktion liefert TRUE.
 *)

FUNCTION AddCookie (id: STRING; value: Anytype): BOOLEAN;
(* Installiert einen Cookie, sucht vorher 'id' und schreibt den Wert
 * value, wenn er schon vorhanden ist.  Ist der Cookie nicht da, Wird er
 * am Ende der Cookieliste angehngt. ACHTUNG!!!  Wenn kein Platz mehr
 * fr den Cookie da ist, oder kein Jar vorhanden ist, wird KEIN NEUER
 * JAR INSTALLIERT!!!
 *)

FUNCTION DelCookie (id: STRING): BOOLEAN;
(* Lscht einen installierten Cookie. Sollte man immer machen, wenn man
 * wrend der Laufzeit einer Applikation einen Cookie installiert hat, und
 * nun terminieren will!!!
 *)


TYPE  Listmode = (first, next);

FUNCTION ListCookies (flag: Listmode; VAR id: STRING; 
                      VAR value: Anytype; VAR num: INTEGER): BOOLEAN;
(* Geht die Liste aller Cookies durch. (Einmal mit first, dann mit next
 * aufrufen). Solange die Prozedur TRUE liefert, sind Cookies gefunden
 * worden. Bei FALSE sind 'id' und 'value' undefiniert!  'num' ist die
 * Nummer des Cookies in der Liste!
 *)

FUNCTION MaxCookies: INTEGER;
(* Anzahl der zur Verfgung stehenden Cookies *)

FUNCTION UsedCookies: INTEGER;
(* Anzahl der zur belegten Cookies *)

FUNCTION JarInstalled: BOOLEAN;
(* TRUE, wenn ein Cookiejar installiert ist. sollte ab TOS 1.06
 * immer der Fall sein...
 *)

IMPLEMENTATION

(*----------------------------------------------------------------------*
 * Int. Vers | Datum    | Name | nderung                               *
 *-----------+----------+------+----------------------------------------*
 *  1.00     | 11.04.92 |  Hp  | Modula-2 to Pascal                     *
 *  1.01     | 21.02.93 |  Hp  | Routinen zum Installieren und Lschen  *
 *           |          |      | von Cookies implementiert              *
 *-----------+----------+------+----------------------------------------*)

USES TOS;

{$I COMPOPT.INC}

CONST   MaxCooks =      1024; (* Mehr Cookies wird ja wohl keiner haben *)

TYPE    tCookie =       RECORD
                         id:  LONGINT;
                         val: LONGINT;
                        END;

TYPE    tJar =          ^Jarlist;
        Jarlist =       ARRAY [0..MaxCooks] OF tCookie;

VAR     dummy:          RECORD
                         CASE BOOLEAN OF
                          TRUE:  (str: ARRAY [0..3] OF CHAR); (* Identifier *)
                          FALSE: (lc: LONGINT);
                        END;

VAR     JarPtr:         tJar ABSOLUTE $05A0;
        Jar:            tJar;
        NoJar:          POINTER;

VAR     NullCookie:     RECORD
                         id:  LONGINT;
                         val: LONGINT;
                        END;

FUNCTION GetJar: LONGINT;
(* Mu im Supervisormode ausgefhrt werden! *)
BEGIN
 Jar:= JarPtr;
 GetJar:= 0;
END (*GetJar*);

FUNCTION FindCookie (id: STRING; VAR value: Anytype): BOOLEAN;
VAR c: INTEGER;
    b: BOOLEAN;
BEGIN
 b:= FALSE;
 Supexec (GetJar);
 IF Jar <> NoJar THEN BEGIN
  FOR c:= 0 TO 3 DO BEGIN
   dummy.str[c]:= id[c+1];
   (* Pascalstrings haben ein fhrendes Lngenbyte!!! *)
  END; 
  c:= 0;
  WHILE NOT b DO BEGIN
   IF Jar^[c].id = dummy.lc THEN BEGIN
    value.long:= Jar^[c].val;  b:= TRUE;
   END ELSE IF (Jar^[c].id = 0) THEN BEGIN
    FindCookie:= FALSE;
    EXIT;
   END;
   INC (c);
  END;
 END;
 FindCookie:= b;
END (*FindCookie*);

FUNCTION ModifyCookie (id: STRING; newvalue: Anytype): BOOLEAN;
VAR c: INTEGER;
    b: BOOLEAN;
BEGIN
 b:= FALSE;
 Supexec (GetJar);
 IF Jar <> NoJar THEN BEGIN
  FOR c:= 0 TO 3 DO BEGIN
   dummy.str[c]:= id[c+1]; (*Pascalstring!*)
  END;
  c:= 0;
  WHILE (Jar^[c].id <> 0) AND NOT b DO BEGIN
   IF Jar^[c].id = dummy.lc THEN BEGIN
    Jar^[c].val:= newvalue.long;  b:= TRUE;
   END;
   INC (c);
  END;
 END;
 ModifyCookie:= b;
END (*ModifyCookie*);

VAR cook: INTEGER;

FUNCTION ListCookies (flag: Listmode; VAR id: STRING; 
                      VAR value: Anytype; VAR num: INTEGER): BOOLEAN;
VAR c: INTEGER;
    ok: BOOLEAN;
BEGIN
 id[0]:= CHR(0);  value.long:= 0;  ok:= FALSE;
 IF flag = first THEN BEGIN  cook:= 0;  END;
 IF cook > -1 THEN BEGIN
  Supexec (GetJar);
  IF Jar <> NoJar THEN BEGIN
   IF Jar^[cook].id <> 0 THEN BEGIN
    value.long:= Jar^[cook].val;  (* Den Cookie-Wert bertragen *)
    dummy.lc:= Jar^[cook].id;  (* den Longcard in einen String wandeln *)
    FOR c:= 0 TO 3 DO BEGIN  id[c+1]:= dummy.str[c];  END;
    id[0]:= CHR(4); (* Pascal!! *)
    num:= cook;
    INC (cook); (* Nchster Cookie *)
    ok:= TRUE;
   END ELSE BEGIN
    id:= 'NULL';
    value.long:= Jar^[cook].val;  (* Den Cookie-Wert bertragen *)
    num:= cook;
    cook:= -1; (* Nchster Cookie *)
    ok:= TRUE;
   END;
  END;
 END;
 ListCookies:= ok;
END (*ListCookies*);

FUNCTION AddCookie (id: STRING; value: Anytype): BOOLEAN;
VAR c, cc: INTEGER;
BEGIN
 IF ModifyCookie (id, value) THEN BEGIN
  AddCookie:= TRUE;  EXIT;
 END;
 Supexec (GetJar);  cc:= 0;
 (* Null-Cookie suchen *)
 WHILE Jar^[cc].id <> 0 DO  INC (cc);
 IF cc < (Jar^[cc].val - 1) THEN BEGIN (* Noch Platz *)
  (* NULL-Cookie umkopieren *)
  Jar^[cc+1].id:= Jar^[cc].id;
  Jar^[cc+1].val:= Jar^[cc].val;
  FOR c:= 0 TO 3 DO BEGIN
   dummy.str[c]:= id[c+1]; (*Pascalstring!*)
  END;
  Jar^[cc].id:= dummy.lc;
  Jar^[cc].val:= value.long;
  AddCookie:= TRUE;  EXIT;
 END;
 AddCookie:= FALSE;
END (*AddCookie*);

FUNCTION DelCookie (id: STRING): BOOLEAN;
VAR c, cc, nc: INTEGER;
    val: Anytype;
BEGIN
 IF NOT FindCookie (id, val) THEN BEGIN
  DelCookie:= FALSE;  EXIT;
 END;
 (* Cookie suchen *)
 FOR c:= 0 TO 3 DO BEGIN
  dummy.str[c]:= id[c+1]; (*Pascalstring!*)
 END;
 Supexec (GetJar);
 (* Den zu lschenden Cookie suchen *)
 cc:= 0;  WHILE Jar^[cc].id <> dummy.lc DO  INC (cc);
 (* Null-Cookie suchen *)
 nc:= 0;  WHILE Jar^[nc].id <> 0 DO  INC (nc);
 WHILE cc < nc DO BEGIN
  Jar^[cc].id:= Jar^[cc+1].id;
  Jar^[cc].val:= Jar^[cc+1].val;
  INC (cc);
 END;
 DelCookie:= TRUE;
END (*DelCookie*);

FUNCTION MaxCookies: INTEGER;
VAR c: INTEGER;
    v: Anytype;
BEGIN
 c:= 0;  v.long:= 0;
 Supexec (GetJar);
 IF Jar <> NoJar THEN BEGIN
  (* Null-Cookie suchen *)
  WHILE Jar^[c].id <> 0 DO  INC (c);
  v.long:= Jar^[c].val;
 END;
 MaxCookies:= v.long;
END (*MaxCookies*);

FUNCTION UsedCookies: INTEGER;
VAR c: INTEGER;
BEGIN
 c:= 0;
 Supexec (GetJar);
 IF Jar <> NoJar THEN BEGIN
  WHILE Jar^[c].id <> 0 DO  INC (c);
 END;
 UsedCookies:= c;
END (*UsedCookies*);

FUNCTION JarInstalled: BOOLEAN;
BEGIN
 Supexec (GetJar);
 JarInstalled:= Jar <> NoJar;
END (*Cookies*);

BEGIN
 cook:= 0;
 NoJar:= POINTER (0);
 NullCookie.id:= 0;
 NullCookie.val:= 0;
END (*M_Cookie*).
